/*
	Resizer plugin for AviSynth -- xxx

	Copyright (C) 2004 Wilbert Dijkhof
  
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation.
	 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
		
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	
	The author can be contacted at:
	Wilbert Dijkhof
	wilbertdijkhof@yahoo.com

	v1.0 10th September 2004 - 1st release
*/

#include "Resizer.h"

Resizer::Resizer(PClip _child, int _width, double _cap_window, int _filmpixel, int _leftborder, const char* _method, IScriptEnvironment* env) : 
	GenericVideoFilter(_child), width(_width), cap_window(_cap_window), filmpixel(_filmpixel), leftborder(_leftborder), method(_method) {
		
		// input parameter checking:
		if ( (width % 16 != 0) || (filmpixel % 2 != 0) || (leftborder % 2 != 0) )
			env->ThrowError("Resizer: filmpixel and leftborder must be even, and width divisible by 16");
		if ( vi.width - filmpixel - leftborder < 0 )
			env->ThrowError("Resizer: width of input clip should be larger or equal than filmpixel + leftborder");
		
		// check resize method
		const char* methodString;
	    if (lstrcmpi(method, "bilinear") == 0) 
			methodString = "BilinearResize";
		else if (lstrcmpi(method, "bicubic") == 0)
			methodString = "BicubicResize";
		else if (lstrcmpi(method, "lanczos") == 0)
			methodString = "LanczosResize";
		else if (lstrcmpi(method, "lanczos4") == 0)
			methodString = "Lanczos4Resize";
		else if (lstrcmpi(method, "point") == 0)
			methodString = "PointResize";
		else 
			env->ThrowError("Resizer: method must be either \"Point\", \"Bilinear\", \"Bicubic\", \"Lanczos\" or \"Lanczos4\"");

		int w = vi.width;
		int h = vi.height;
		// calculation pal_width
		int pal_width = int((52. / cap_window) * w);
		// YUV pal_width should be even
		if (pal_width % 2 != 0) {
			pal_width = pal_width + 1;
		}
		
		// black border on the left: leftborder
		// black border on the right: w - filmpixel - leftborder
		// total black border: w - filmpixel
		int rightborder = w - filmpixel - leftborder;

		if (pal_width < w) { // crop difference between w and pal_width
			// (w-pal_width)/(w-filmpixel) * leftborder -> left croppen
			// (w-pal_width)/(w-filmpixel) * rightborder -> right croppen
			int leftcrop = int( ((w-pal_width)*leftborder)/(w-filmpixel) );
			if (leftcrop % 2 != 0) { // leftcrop should be even
				leftcrop = leftcrop + leftcrop % 2;
			}
			int rightcrop = int( ((w-pal_width)*rightborder)/(w-filmpixel) );
			if (rightcrop % 2 != 0) { // rightcrop should be even
				rightcrop = rightcrop + rightcrop % 2;
			}
			AVSValue cr_args[5] = { child, leftcrop, 0, -rightcrop, 0 };
			cropped = env->Invoke("Crop", AVSValue(cr_args,5)).AsClip();
			vi.width = w - leftcrop - rightcrop;
		} else if (pal_width > w) { // addborders w -> pal_width
			int leftadd = int(0.5*(pal_width-w) + 0.5*(rightborder-leftborder));
			if (leftadd % 2 != 0) { // leftadd should be even
				leftadd = leftadd + leftadd % 2;
			}
			int rightadd = int(0.5*(pal_width-w) + 0.5*(leftborder-rightborder));
			if (rightadd % 2 != 0) { // rightadd should be even
				rightadd = rightadd + rightadd % 2;
			}
			AVSValue bo_args[5] = { child, leftadd, 0, rightadd, 0 };
			cropped = env->Invoke("AddBorders", AVSValue(bo_args,5)).AsClip();
		} else {
			cropped = child;
		}
		//
		AVSValue up_args[3] = { cropped, width, int(0.75*width) };
		resized = env->Invoke(methodString, AVSValue(up_args,3)).AsClip();
		vi.width = width;
		vi.height = int(0.75*width);
	}

Resizer::~Resizer() {
}
	
PVideoFrame __stdcall Resizer::GetFrame(int n, IScriptEnvironment* env) {

	//PVideoFrame src = child->GetFrame(n, env);
	PVideoFrame test = resized->GetFrame(n, env);
	//PVideoFrame dst = env->NewVideoFrame(vi);

	const int test_pitch = test->GetPitch();
	const int test_width = test->GetRowSize();
	const int test_height = test->GetHeight();

	//const int dst_pitch = dst->GetPitch();
	//const int dst_width = dst->GetRowSize();
	//const int dst_height = dst->GetHeight();

	const unsigned char* testp = test->GetReadPtr();
	//unsigned char* dstp = dst->GetWritePtr();

	return test;
}

AVSValue __cdecl Create_Resizer(AVSValue args, void* user_data, IScriptEnvironment* env) {
    return new Resizer(args[0].AsClip(),
			args[1].AsInt(640),
			args[2].AsFloat(52),
			args[3].AsInt(640),
			args[4].AsInt(0),
			args[5].AsString("bicubic"),
			env);
}


extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {
	env->AddFunction("Resizer", "c[width]i[cap_window]f[filmpixel]i[leftborder]i[method]s", Create_Resizer, 0);
    return "`Resizer' Resizer plugin";
}
